var _sizeConversor;

import { useState, useEffect, forwardRef } from 'react';
import PropTypes from 'prop-types';
import cx from 'classnames';
import AtomButton, { atomButtonSizes } from '@s-ui/react-atom-button';
import AtomInput, { inputSizes } from '@s-ui/react-atom-input';
import MoleculeField from '@s-ui/react-molecule-field';
import { ACTIONS } from './config';
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
var BUTTON_TYPE = 'secondary';
var BASE_CLASS = "sui-MoleculeDataCounter";
var CLASS_INPUT_CONTAINER = BASE_CLASS + "-container";
var sizeConversor = (_sizeConversor = {}, _sizeConversor[inputSizes.SMALL] = atomButtonSizes.SMALL, _sizeConversor[inputSizes.LARGE] = atomButtonSizes.LARGE, _sizeConversor);
var MoleculeDataCounter = /*#__PURE__*/forwardRef(function (_ref, ref) {
  var _ref$addIcon = _ref.addIcon,
      addIcon = _ref$addIcon === void 0 ? '+' : _ref$addIcon,
      charsSize = _ref.charsSize,
      disabled = _ref.disabled,
      errorTextProps = _ref.errorText,
      id = _ref.id,
      _ref$inputDisabled = _ref.inputDisabled,
      inputDisabled = _ref$inputDisabled === void 0 ? false : _ref$inputDisabled,
      _ref$isLoading = _ref.isLoading,
      isLoading = _ref$isLoading === void 0 ? false : _ref$isLoading,
      label = _ref.label,
      _ref$max = _ref.max,
      max = _ref$max === void 0 ? 100 : _ref$max,
      maxValueErrorText = _ref.maxValueErrorText,
      maxValueHelpText = _ref.maxValueHelpText,
      _ref$min = _ref.min,
      min = _ref$min === void 0 ? 0 : _ref$min,
      minValueErrorText = _ref.minValueErrorText,
      minValueHelpText = _ref.minValueHelpText,
      onChange = _ref.onChange,
      _ref$size = _ref.size,
      size = _ref$size === void 0 ? inputSizes.MEDIUM : _ref$size,
      _ref$substractIcon = _ref.substractIcon,
      substractIcon = _ref$substractIcon === void 0 ? '-' : _ref$substractIcon,
      initialValue = _ref.initialValue,
      value = _ref.value;
  var initialStateValue;
  var numMax = Number(max);
  var numMin = Number(min);

  if (value !== undefined) {
    initialStateValue = Number(value);
  } else if (initialValue !== undefined) {
    initialStateValue = Number(initialValue);
  } else {
    initialStateValue = Math.trunc((numMax - numMin) / 2);
  }

  var _useState = useState(),
      lastAction = _useState[0],
      setLastActions = _useState[1];

  var _useState2 = useState(initialStateValue),
      internalValue = _useState2[0],
      setInternalValue = _useState2[1];

  useEffect(function () {
    if (value !== undefined) {
      setInternalValue(Number(value));
    }
  }, [value]);

  var isMaxValue = function isMaxValue(number) {
    if (number === void 0) {
      number = internalValue;
    }

    return number === numMax;
  };

  var isMinValue = function isMinValue(number) {
    if (number === void 0) {
      number = internalValue;
    }

    return number === numMin;
  };

  var isLowerThanMinValue = function isLowerThanMinValue(number) {
    if (number === void 0) {
      number = internalValue;
    }

    return number < numMin;
  };

  var isHigherThanMaxValue = function isHigherThanMaxValue(number) {
    if (number === void 0) {
      number = internalValue;
    }

    return number > numMax;
  };

  var decrementDisabled = disabled || internalValue <= numMin;
  var incrementDisabled = disabled || internalValue >= numMax;

  var assignValue = function assignValue(event, _ref2) {
    var newValue = _ref2.newValue,
        lastAction = _ref2.lastAction;

    if (value === undefined) {
      // uncontrolled component
      setInternalValue(newValue);
    }

    if (lastAction) {
      setLastActions(lastAction);
    }

    if (typeof onChange === 'function') {
      onChange(event, {
        value: String(newValue),
        action: lastAction
      });
    }
  };

  var incrementValue = function incrementValue(event) {
    var newValue = internalValue + 1;

    if (isHigherThanMaxValue(newValue)) {
      newValue = internalValue;
    }

    assignValue(event, {
      newValue: newValue,
      lastAction: ACTIONS.MORE
    });
  };

  var decrementValue = function decrementValue(event) {
    var newValue = internalValue - 1;

    if (isLowerThanMinValue(newValue)) {
      newValue = internalValue;
    }

    assignValue(event, {
      newValue: newValue,
      lastAction: ACTIONS.LESS
    });
  };

  var handleChange = function handleChange(event, _ref3) {
    var value = _ref3.value;
    var newValue = parseInt(value, 10);
    assignValue(event, {
      newValue: isNaN(newValue) ? '' : newValue,
      lastAction: ACTIONS.CHANGE
    });
  };

  var handleKeyDown = function handleKeyDown(event) {
    var key = event.key;
    if (key === 'ArrowUp') incrementValue(event);
    if (key === 'ArrowDown') decrementValue(event);
  };

  var helpText, errorText;

  if (!disabled) {
    if (isMaxValue()) helpText = maxValueHelpText;else if (isMinValue()) helpText = minValueHelpText;else helpText = null;
    if (isLowerThanMinValue()) errorText = minValueErrorText;else if (isHigherThanMaxValue()) errorText = maxValueErrorText;else if (errorTextProps) errorText = errorTextProps;else errorText = null;
  }

  return /*#__PURE__*/_jsx("div", {
    className: BASE_CLASS,
    children: /*#__PURE__*/_jsx(MoleculeField, {
      name: id,
      label: label,
      helpText: helpText,
      errorText: errorText,
      children: /*#__PURE__*/_jsxs("div", {
        className: cx(CLASS_INPUT_CONTAINER, CLASS_INPUT_CONTAINER + "--" + size),
        children: [/*#__PURE__*/_jsx(AtomButton, {
          disabled: decrementDisabled,
          isLoading: isLoading && lastAction === ACTIONS.LESS,
          onClick: decrementValue,
          size: sizeConversor[size],
          type: BUTTON_TYPE,
          children: substractIcon
        }), /*#__PURE__*/_jsx(AtomInput, {
          ref: ref,
          charsSize: charsSize || internalValue.toString().length <= 2 ? 2 : internalValue.toString().length,
          disabled: disabled || inputDisabled,
          id: id,
          isLoading: isLoading && lastAction === ACTIONS.CHANGE,
          onChange: handleChange,
          onKeyDown: handleKeyDown,
          size: size,
          value: internalValue
        }), /*#__PURE__*/_jsx(AtomButton, {
          disabled: incrementDisabled,
          isLoading: isLoading && lastAction === ACTIONS.MORE,
          onClick: incrementValue,
          size: sizeConversor[size],
          type: BUTTON_TYPE,
          children: addIcon
        })]
      })
    })
  });
});
MoleculeDataCounter.displayName = 'MoleculeDataCounter';
export default MoleculeDataCounter;
export { inputSizes as moleculeDataCounterSizes };